/**
 * http://www.wayforward.net/avGPS/libav/av_mem.c
 * Copyright(c) 2005, 2008 Wayforward LLC
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
#include <errno.h>

#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/fcntl.h>

#include <unistd.h>

#include "av_mem.h"
#include "av_err.h"

static const char OPEN_FN[] = "open";
static const char FSTAT_FN[] = "fstat";
static const char MMAP_FN[] = "mmap";

int
map_file(const char *filename, struct mapped_file *pmf) {
	int fd = open(filename, O_RDONLY);

	if (fd == -1) {
		err_push_errno(OPEN_FN, filename);
	}
	else {
		struct stat sb;

		if (fstat(fd, &sb) == -1) {
			err_push_errno(FSTAT_FN, filename);
		}
		else {
			const void *pv;

			pv = mmap(NULL, sb.st_size, PROT_READ,
				  MAP_FILE | MAP_PRIVATE, fd, 0);

			if (pv == (const void *) -1) {
				err_push_errno(MMAP_FN, filename);
			}
			else {
				pmf->fd = fd;
				pmf->len = sb.st_size;
				pmf->pv = pv;
				return 1;
			}
		}

		close(fd);
	}

	return 0;
}

void
unmap_file(struct mapped_file *pmf) {
	(void) munmap((void *) pmf->pv, pmf->len);
	(void) close(pmf->fd);
}

/*
 * Overrides for Emacs so that we follow Linux's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the
 * end of the file.
 * ----------------------------------------------------------------------
 * Local Variables:
 * c-file-style: "linux"
 * End:
 */
